/**
 * @description Excute step function with requested payload
 *  Workflow engine step 
 *  STEP-1 : Read the payload 
 *  STEP-2 : Analyze payload and run buisness logic
 *  STEP-3 : Store payload in S3 if is_store_result = 1
 */
const Joi = require('@hapi/joi');
const { constants, errorHandlers, commonMethods } = require('./common');
const Assert = require("assert");

class WorflowStep {
    /**
     * @description payload from workflow engine
     * @param {StepExecutionService} stepExecutionService Instance of service
     * @param {Object} payload Payload to run workflow step lambda
     */
    constructor(stepExecutionService, payload) {
        Assert(stepExecutionService, 'stepExecutionService is mandatory');
        Assert(payload, 'payload is mandatory');
        this.stepExecutionService = stepExecutionService;
        this.payload = payload;
        const _payloadSchema = this.getReportSchema();
        // const _validPayload = _payloadSchema.validate(payload);
        // if (_validPayload.error) {
        //     throw _validPayload.error;
        // }
    }

    /** 
     * @description Get success response of lambda
     * @return object for success response
     * @param {Object} data - Body data for response
     * @private
    */
    getSuccessObject(data) {
        return {
            body: data,
            message: constants.STATUS.SUCCESS,
            statusCode: 200
        }
    }

    /**
     * @description Get Report schema
     */
    getReportSchema() {
        return Joi.object().keys({
            reportId: Joi.alternatives().try(Joi.string(), Joi.number()).required(),
            reportId: Joi.alternatives().try(Joi.string(), Joi.number()).required(),
            data: Joi.object().optional(),
            data1: Joi.object().optional(),
            s3Object: Joi.object().optional()
        });
    }

    /**
     * @description Update current payload with persisted data1
     * @param {JSON} persistedPayload 
     */
    updateCurrentPayload(persistedPayload) {
        this.payload.data1 = persistedPayload.data1;
    }

    /**
     * @description execute current step
     */
    async executeCurrentStep() {
        // Initiate return data
        let _returnData = {
            response: null,
            data: {}
        };
        try {  
            // Get payload data for current step
            let _payloadData = await this.stepExecutionService.getPersistPayloadCurrentStep(this.payload.reportId) || this.payload;
            //  Initialize data1 if not available
            _payloadData.data1 = _payloadData.data1 || {};
            //get reportId from payload
            const _reportId = this.payload.reportId;
            const _newJobNumTemp = _payloadData.data1.oldData ? _payloadData.data1.oldData.value : '';
            const _newJobNumber = this.stepExecutionService.getNewJobNumber(_reportId, _newJobNumTemp);

            if (_newJobNumber) {
                 // Prepare report
                _payloadData = this.stepExecutionService.prepareReportDataToUpdate(_reportId, _payloadData,
                    _newJobNumber);
            }
               
            this.updateCurrentPayload(_payloadData);
            _returnData.response = this.getSuccessObject(constants.RESULT_SUCCESS);
        } catch (err) {
            _returnData.response = !err.statusCode ? errorHandlers.getErrorResponse(err.stack, err.message)
                : err;
        }
        return _returnData.response;
    }

    /**
     * @description Save payload in s3 to execute engine
     * @param {Object} response 
     */
    async savePayloadToExecuteEngine(response) {
        let _response = response;
        // Exceptional handling for storing payload back to s3
        try {
            // Prepare payload to store s3
            const _payloadToSave = commonMethods.prepareResponsePayload(response, this.payload);
            // Save payload will be done after try and catch
            await this.stepExecutionService.savePayloadToExecuteEngine(_payloadToSave);
        } catch (err) {
            _response = !err.statusCode ? errorHandlers.getErrorResponse(err.stack, err.message)
                : _response = err;
        }
        return _response;
    }
}

module.exports = WorflowStep;