const { errorHandlers, constants } = require('./../common');
const config = require('../../config/index');
const Assert = require('assert');

// workflow helper

/**
 * @description read json and execute next step
 * Step execution service
 * 1. Read S3 JSON data
 * 2. Execute required step
 */
class stepExecutionService {

  // Initailize services and necessary data members  
  constructor(payload, queryBuilder, WorkflowHelper, utilityService) {
    Assert(payload, 'payload is mandatory');
    this.s3Object = payload.s3Object || {};
    this.payload = payload;
    this.queryBuilder = queryBuilder;
    this.workflowHelper = WorkflowHelper;
    this.utilityService = utilityService;
  }

  /**
   * @description Get report data
   * @param {number} reportId 
   */
  async getReportMasterData(reportId) {
    try {
      const _getReportQuery = this.queryBuilder.buildGetQuery(`${constants.DB_PREFIX}.${constants.EP_MASTER_VIEW}`, reportId);
      const _queryResult = await this.workflowHelper.executeQuery(_getReportQuery);
      if (_queryResult.rows && _queryResult.rows.length) {
        return _queryResult.rows[0];
      } else {
        throw errorHandlers.getReportErrorHandler(constants.REPORT_ERROR);
      }
    } catch (error) {
      throw errorHandlers.getReportErrorHandler(error.stack);
    }
  }

  /**
   * @description Check is job number temporary or not
   * @param {String} jobNumber
   */
  isTempJobNumber(jobNumber) {
    return this.utilityService.isTempJobNumber(jobNumber);
  }

  /**
   * @description Check is job number temporary or not
   * @param {String} jobNumber
   */
  getJobNumber(departmentCode, isTemp) {
    return this.utilityService.getJobNumber(departmentCode, isTemp);
  }

  /**
   * @description Return new job number and move file to s3 
   * @param {String} reportId
   * @param {String} newJobNumTemp
   */
  async getNewJobNumber(reportId, newJobNumTemp) {
    const _reportMasterData = await this.getReportMasterData(reportId);
    const { region_name, business_unit_abbreviation, report_type_short_name, job_num, job_number_temp, report_guid, department_pk } = _reportMasterData;
    const filepath = `${config.FILE_PATH_PREFIX}${region_name}/Job/${business_unit_abbreviation}/${report_type_short_name}`;
    if (this.isTempJobNumber(job_num)) {
      if (newJobNumTemp) {
        return newJobNumTemp;
      } else {
        const _newJobNumber = await this.getJobNumber(department_pk, false);
        const s3Object = {
          sourceBucketName: config.PRIVATE_BUCKET_NAME,
          destinationBucketName: config.PRIVATE_BUCKET_NAME,
          sourceKey: `${filepath}/${job_num} ${report_guid}/`,
          destinationKey: `${filepath}/${_newJobNumber}/`
        };
        await this.workflowHelper.copyS3Files(s3Object, true);
        return newJobNumber;
      }
    } else {
      const s3Object = {
        sourceBucketName: config.PRIVATE_BUCKET_NAME,
        sourceKey: `${filepath}/${job_number_temp}/`,
      };
      await this.workflowHelper.deleteS3Files(s3Object, true);
    }
  }

  /**
   * @description Get persist payload for current step
   * @param {String} reportId
   */
  async getPersistPayloadCurrentStep(reportId) {
    return this.workflowHelper.getPeristPayloadData(reportId);
  }

  /**
   * @description Update report data
   * @param {JSON} reportData 
   */
  // async updateReportData(reportData) {
  //   if (reportData.cellData.length) {
  //     const _updateQuery = this.queryBuilder.buildUpdateQuery(`${constants.DB_PREFIX}.${constants.EP_MASTER_TABLE}`,
  //       reportData.cellData, reportData.conditions);
  //     await this.workflowHelper.executeQuery(_updateQuery);
  //   }
  // }

  /**
   * @description Add history
   */
  // async addHistory(historyData) {
  //   try {
  //     const _historyFuncQuery = this.queryBuilder.buildHistoryFuncQuery(`${constants.DB_PREFIX}.${constants.EP_HISTORY_FUNCTION}`,
  //       historyData);
  //     await this.workflowHelper.executeQuery(_historyFuncQuery);
  //   } catch (error) {
  //     throw errorHandlers.insertHistoryForReport(error.stack);
  //   }
  // }

  /**
   * @description Insert PDF data
   * @param {Array} pdfData 
   */
  // async insertPdfData(pdfData) {
  //   try {
  //     const _insertPDFQuery = this.queryBuilder.buildPDFQuery(`${constants.DB_PREFIX}.${constants.EP_REPORT_PDF_TABLE}`,
  //       pdfData.cellData, constants.EP_PDF_QUERY);
  //     await this.workflowHelper.executeQuery(_insertPDFQuery);
  //   } catch (error) {
  //     throw errorHandlers.insertPDFForReport(error.stack);
  //   }
  // }

  /**
 * @description append default condition if master_pk is not available
 * @param {number} reportId 
 * @param {JSON} _updateReportData 
 */
  appendDefaultCondition(reportId, _updateReportData) {
    const _tempReportData = _updateReportData;
    const _conditions = _tempReportData.conditions || [];
    let _masterPkConditionFlag = false;
    for (let i = 0; i < _conditions.length; i++) {
      if (_conditions[i].column === constants.MASTER_PK) {
        _masterPkConditionFlag = true;
      }
    }
    if (!_masterPkConditionFlag) {
      _conditions.push({
        "column": constants.MASTER_PK,
        "value": reportId,
        "type": "number"
      });
    }
    _tempReportData.conditions = _conditions;
    return _tempReportData;
  }

  /**
   * @description Prepare history for status 
   * @param {string} reportId 
   * @param {JSON} statusData 
   */
  // prepareHistoryData(reportId, statusData) {
  //   return [{
  //     "column": "report_id",
  //     "value": reportId,
  //     "type": "string"
  //   }, {
  //     "column": "operation_pk",
  //     "value": statusData.operation_pk,
  //     "type": "number"
  //   }, {
  //     "column": "status_config_pk",
  //     "value": statusData[constants.STATUS_CONFIG],
  //     "type": "number"
  //   }, {
  //     "column": "to_status_pk",
  //     "value": statusData.to_status,
  //     "type": "number"
  //   }, {
  //     "column": "from_status_pk",
  //     "value": statusData.from_status,
  //     "type": "number"
  //   }];
  // }

  /**
   * @description Prepare master data
   * @param {string} reportId 
   * @param {JSON} payload
   * @param {string} newJobNumber
   */
  prepareReportDataToUpdate(reportId, payload, newJobNumber) {
    const _initialPayload = payload;
    _initialPayload.data1 = payload.data1 || {};
    _initialPayload.data2 = payload.data2 || {};
    let _updateReportData = payload.data1 && payload.data1.updateReportData ? payload.data1.updateReportData : {
      cellData: []
    };
    _updateReportData = this.appendDefaultCondition(reportId, _updateReportData);
    let _jobNumColFound = false;
    // Prepare cell data
    for (let i = 0; i < _updateReportData.cellData.length; i++) {
      if (_updateReportData.cellData[i].column === "job_num") {
        _updateReportData.cellData[i].value = newJobNumber;
        _jobNumColFound = true;
      }
    }
    if (!_jobNumColFound) {
      _updateReportData.cellData.push({
        "column": "job_num",
        "value": newJobNumber,
        "type": "string"
      });
    }
    // Update payload to send
    _initialPayload.data1.updateReportData = _updateReportData;
    _initialPayload.data1 = {
      "oldData": {
        "column": "new_job_number_temp",
        "value": newJobNumber,
        "type": "string"
      }
    };
    return _initialPayload;
  }

  /**
 * @description Get next status of current report as per the operation
 * @param {String} operation 
 * @param {String} reportId 
 */
  // async getNextStatusOfReport(reportId, operation) {
  //   let _result = null;
  //   try {
  //     const _function = `${constants.DB_PREFIX}.${constants.GET_NEXT_STATUS}`;
  //     const _response = await this.workflowHelper.executeFunction(_function, [reportId, operation]);
  //     if (_response.rowCount && _response.rows[0]) {
  //       _result = _response.rows[0];
  //     }
  //   } catch (err) {
  //     throw errorHandlers.getStatusErrorHandler(err.stack);
  //   }
  //   return _result;
  // }

  /**
   * @description Check status_flag in operation table
   * @param {string} operation_pk 
   */
  // async checkStatusFlag(operation_pk) {
  //   let _status_update_flag = false;
  //   const _function = `${constants.DB_PREFIX}.${constants.EP_CHECK_STATUS}`;
  //   const _response = await this.workflowHelper.executeFunction(_function, [operation_pk]);
  //   if (_response.rowCount) {
  //     _status_update_flag = _response.rows[0][constants.EP_CHECK_STATUS];
  //   }
  //   return _status_update_flag;
  // }

  /**
   * @description Save payload to execute workflow-engine
   * @param {Object} payload
   */
  async savePayloadToExecuteEngine(payload) {
    let _location = null;
    try {
      this.logExecutionStep('START - Upload payload to s3', payload);
      // Save payload in s3 which is expected by workflow engine
      const _s3Response = await this.workflowHelper.putPayloadData(this.s3Object, payload);
      _location = _s3Response.Location;
      this.logExecutionStep('END -  Upload payload to s3');
    } catch (err) {
      throw errorHandlers.savePayloadErrorHandler(err.stack);
    }
    return _location;
  }

  /**
   * @description All console.logs at a single place
   */
  logExecutionStep(message, value) {
    !value ? console.log(`--${message}--`) : console.log(`--${message}--`, value);
  }
}

module.exports = stepExecutionService;