'use strict';
const sinon = require('sinon');
const { expect } = require('chai');
const WorflowStep = require('./../src');
const { S3Service, DatabaseService, StepExecutionService, Repository } = require('./../src/service');
const { QueryBuilder } = require('./../src/common');


/**
 * @description Test cases for WorkflowStep
 */
describe('Class: WorflowStep:', () => {
    const stubError = 'some error occurred.';
    const stepExecutionService = {
        getNextStatusOfReport: () => { },
        updateStatusForReport: () => { },
        savePayloadAndRunNextStep: () => { },
        updatePayloadAndSave: () => { }
    };
    const samplePayload = {
        "result": "audit engine",
        "data": "",
        "reportId": "2",
        "data1": "",
        "s3Object": {
            "Key": "",
            "Bucket": ""
        }
    };
    const worflowStep = new WorflowStep(stepExecutionService, samplePayload);
    // const service = new Service({ repository });
    const sandbox = sinon.createSandbox();
    // Restore stub sandbox
    afterEach('restore the sandbox', () => {
        sandbox.restore();
    });
    // Constructor should send error
    describe("construction : ", () => {
        // Check both parameters
        it("should throw when it is constructed without stepExecutionService", () => {
            expect(() => new WorflowStep()).to.throw();
        });
        it("should throw when it is constructed without payload", () => {
            expect(() => new WorflowStep(stepExecutionService)).to.throw();
        });
    });
    // Check methods
    describe('Method: executeCurrentStep: ', () => {
        // exeception cases
        it('should throw error when getNextStatusOfReport from DB throws exception', async () => {
            sandbox.stub(stepExecutionService, 'getNextStatusOfReport').rejects(new Error(stubError));
            try {
                await worflowStep.executeCurrentStep();
            } catch (error) {
                expect(error.message).to.equal(stubError);
            }
        });
        it('should throw error when updateStatusForReport from db throws exception', async () => {
            sandbox.stub(stepExecutionService, 'updateStatusForReport').rejects(new Error(stubError));
            try {
                await worflowStep.executeCurrentStep();
            } catch (error) {
                expect(error.message).contains('mapping not found in database');
            }
        });

        it('Should run successfully when correct data came from getNextStatusOfReport', async () => {
            sandbox.stub(stepExecutionService, 'getNextStatusOfReport').resolves('2');
            sandbox.stub(stepExecutionService, 'updateStatusForReport').resolves();
            try {
                await worflowStep.executeCurrentStep();
            } catch (error) {
                expect(error.message).to.equal(stubError);
            }
        });
        it('should throw error when updateStatusForReport from db throws exception', async () => {
            sandbox.stub(stepExecutionService, 'updateStatusForReport').rejects(new Error(stubError));
            try {
                await worflowStep.executeCurrentStep();
            } catch (error) {
                expect(error.message).contains('mapping not found in database');
            }
        });
    });
});

describe('Class: stepExecutionService:', () => {
    const stubError = 'some error occurred.';
    const s3Service = {
        putPayloadData: () => { }
    };
    const databaseService = {
        getNextStatusOfReport: () => { },
        updateStatusForReport: () => { }
    };
    const samplePayload = {
        "result": "audit engine",
        "data": "",
        "reportId": "2",
        "data1": "",
        "s3Object": {
            "Key": "",
            "Bucket": ""
        }
    };
    const stepExecutionService = new StepExecutionService(samplePayload, s3Service, databaseService);
    // const service = new Service({ repository });
    const sandbox = sinon.createSandbox();
    // Restore stub sandbox
    afterEach('restore the sandbox', () => {
        sandbox.restore();
    });
    // Constructor should send error
    describe("construction : ", () => {
        it("should throw when it is constructed without payload", () => {
            expect(() => new stepExecutionService()).to.throw();
        });
        it("should throw when it is constructed without s3Service", () => {
            expect(() => new stepExecutionService(samplePayload)).to.throw();
        });
        it("should throw when it is constructed without databaseService", () => {
            expect(() => new stepExecutionService(samplePayload, s3Service)).to.throw();
        });
    });

    // Check methods
    describe('Method: getNextStatusOfReport: ', () => {
        it('should throw error when getNextStatusOfReport from DB throws exception', async () => {
            sandbox.stub(databaseService, 'getNextStatusOfReport').rejects(new Error(stubError));
            try {
                await stepExecutionService.getNextStatusOfReport('', '');
            } catch (error) {
                expect(error.message).contains('GET NEXT STATUS FOR REPORT');
            }
        });
    });

    describe('Method: updateStatusForReport: ', () => {
        it('should throw error when updateStatusForReport from DB throws exception', async () => {
            sandbox.stub(databaseService, 'updateStatusForReport').rejects(new Error(stubError));
            try {
                await stepExecutionService.updateStatusForReport('', '');
            } catch (error) {
                expect(error.message).contains('UPDATE REPORT STATUS FAILED');
            }
        });
    });

    describe('Method: savePayloadAndRunNextStep: ', () => {
        it('should throw error when savePayloadAndRunNextStep from DB throws exception', async () => {
            sandbox.stub(s3Service, 'putPayloadData').rejects(new Error(stubError));
            try {
                await stepExecutionService.updateStatusForReport('', '');
            } catch (error) {
                expect(error.message).contains('UPDATE REPORT STATUS FAILED');
            }
        });
    });

});

describe('Class: DatabaseService:', () => {
    const stubError = 'some error occurred.';
    const queryBuilder = {
        selectFunction: () => { }
    };
    const repository = {
        query: () => { }
    };
    const samplePayload = {
        "result": "audit engine",
        "data": "",
        "reportId": "2",
        "data1": "",
        "s3Object": {
            "Key": "",
            "Bucket": ""
        }
    };

    const databaseService = new DatabaseService(repository, queryBuilder);
    // const service = new Service({ repository });
    const sandbox = sinon.createSandbox();
    // Restore stub sandbox
    afterEach('restore the sandbox', () => {
        sandbox.restore();
    });
    describe('Method: getNextStatusOfReport: ', () => {
        it('Should throw exeception when wrong data came as output', async () => {
            const _resultObj = null;
            sandbox.stub(repository, 'query').resolves(_resultObj);
            try {
                await databaseService.getNextStatusOfReport('2', 'test');
            } catch (err) {
                expect(err.message).contains(`Cannot read property 'rowCount' of null`);
            }
        });
        it('should run successfully without exception', async () => {
            const _resultObj = {
                rowsCount: 1,
                rows: [
                    {
                        "ep_get_next_status_report": null
                    }
                ]
            };
            sandbox.stub(repository, 'query').resolves(_resultObj);
            const _toId = await databaseService.getNextStatusOfReport('2', 'test');
            expect(_toId).to.be.equal(null);
        });
    });

    describe('Method: updateStatusForReport: ', () => {
        it('Should throw exeception when wrong data came as output', async () => {
            const _resultObj = null;
            sandbox.stub(repository, 'query').resolves(_resultObj);
            try {
                await databaseService.getNextStatusOfReport('2', 'test');
            } catch (err) {
                expect(err.message).contains(`Cannot read property 'rowCount' of null`);
            }
        });
        it('should run successfully without exception', async () => {
            const _resultObj = {
                rowsCount: 1,
                rows: [
                    {
                        "ep_get_next_status_report": null
                    }
                ]
            };
            sandbox.stub(repository, 'query').resolves(_resultObj);
            const _toId = await databaseService.getNextStatusOfReport('2', 'test');
            expect(_toId).to.be.equal(null);
        });
    });
});